defmodule Forum.Post do
  use Ecto.Schema
  import Ecto.Changeset

  alias Forum.Email
  alias Forum.Mailer
  alias Forum.Post

  schema "posts" do
    field(:content, :string)

    belongs_to(:user, Forum.User)
    belongs_to(:thread, Forum.Thread)
    belongs_to(:parent, Forum.Post)
    has_many(:children, Forum.Post)

    timestamps()
  end

  @doc false
  def changeset(%Post{} = post, attrs) do
    post
    |> cast(attrs, [:content, :thread_id, :user_id])
    |> validate_required([:content, :thread_id, :user_id])
    |> foreign_key_constraint(:user_id)
    |> foreign_key_constraint(:thread_id)
  end

  @doc """
  Find ~mentions in post content
  """
  def find_mentions(content) do
    ~r/~(\w+)\b/
    |> Regex.scan(content)
    |> Enum.map(fn m -> List.last(m) end)
  end

  @doc """
  Handle mentions, notify users.
  Send email to users
  """
  def handle_mentions(post) do
    for user <- find_mentions(post.content) do
      case Forum.get_user(user) do
        nil ->
          inspect(user)

        mentioned_user ->
          mentioned_user
          |> Email.mention(Forum.Repo.preload(post, [:user, :thread]))
          |> Mailer.deliver_later()
      end
    end

    "stuff"
  end
end
