defmodule ForumWeb.ThreadController do
  use ForumWeb, :controller

  import Ecto.Query

  alias Forum.Post
  alias Forum.Repo
  alias Forum.Thread
  alias Forum.User

  def index(conn, params) do
    stickies =
      Thread
      |> order_by(desc: :updated_at)
      |> where(sticky: true)
      |> preload(:user)
      |> Repo.all()

    {threads, kerosene} =
      Thread
      |> order_by(desc: :id)
      |> where(sticky: false)
      |> preload(:user)
      |> Repo.paginate(params)

    {:ok, response} = Tesla.get("https://tilde.team/~ben/api/")

    render(
      conn,
      "index.html",
      stickies: stickies,
      threads: threads,
      kerosene: kerosene,
      tagline: response.body
    )
  end

  def new(conn, _params) do
    changeset =
      Forum.change_thread(%Thread{
        user_id: conn |> Util.current_username() |> Forum.get_user_id_by_name()
      })

    render(conn, "new.html", changeset: changeset)
  end

  def create(conn, %{"thread" => thread_params}) do
    case Forum.create_thread(thread_params) do
      {:ok, thread} ->
        conn
        |> put_flash(:info, "thread created successfully.")
        |> redirect(to: thread_path(conn, :show, thread))

      {:error, %Ecto.Changeset{} = changeset} ->
        render(conn, "new.html", changeset: changeset)
    end
  end

  def show(conn, %{"id" => id} = params) do
    {posts, kerosene} =
      Post
      |> where(thread_id: ^id)
      |> preload(:user)
      |> Repo.paginate(params)

    thread = Forum.get_thread!(id)

    current_user =
      User
      |> where(name: ^Util.current_username(conn))
      |> select([:id])
      |> Repo.one()

    render(
      conn,
      "show.html",
      thread: thread,
      posts: Enum.with_index(posts),
      kerosene: kerosene,
      user_id: current_user.id,
      page_title: "#{thread.name} // forum.tilde.team"
    )
  end

  def edit(conn, %{"id" => id}) do
    thread = Forum.get_thread!(id)

    if Util.can_edit?(conn, thread.user) do
      render(
        conn,
        "edit.html",
        thread: thread,
        changeset: Forum.change_thread(thread),
        user_id: thread.user.id
      )
    else
      conn
      |> put_flash(:error, "you can't edit a thread you didn't start, silly")
      |> redirect(to: thread_path(conn, :show, thread))
    end
  end

  def update(conn, %{"id" => id, "thread" => thread_params}) do
    thread = Forum.get_thread!(id)

    case Forum.update_thread(thread, thread_params) do
      {:ok, thread} ->
        conn
        |> put_flash(:info, "thread updated successfully.")
        |> redirect(to: thread_path(conn, :show, thread))

      {:error, %Ecto.Changeset{} = changeset} ->
        render(conn, "edit.html", thread: thread, changeset: changeset)
    end
  end

  def delete(conn, %{"id" => id}) do
    thread = Forum.get_thread!(id)
    {:ok, _thread} = Forum.delete_thread(thread)

    conn
    |> put_flash(:info, "thread deleted successfully.")
    |> redirect(to: thread_path(conn, :index))
  end
end
